require 'dp_stm_map'
require 'timeout'
require 'tmpdir'
require 'fileutils'
require "xray/thread_dump_signal_handler"

module DpStmMap



  describe ClientLocalStore do


     before do
      @storage_dir=Dir.mktmpdir
    end

    after do
      FileUtils.rm_rf(@storage_dir)
    end

    subject {ClientLocalStore.new @storage_dir}

    describe :update do
      it "should store new value" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set)
        subject['x'].should == 'value'
      end
      it "should update existing value" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set)
        subject.update(1,{'def' => "changed"}, {'x' => 'def'}, [].to_set)
        subject['x'].should == 'changed'
      end
      it "should raise exception if content is not found" do
        expect{subject.update(1,{}, {'x' => 'dd'}, [].to_set)}.to raise_error StandardError
      end

      it "should delete values when new hash is nil" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set)
        subject.update(1,{}, {'x' => nil}, ['abc'].to_set)
        subject['x'].should be_nil
      end

      it "should report changes caused by the update" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set).should == {'x' => [nil,'value']}
      end

    end

    describe :has_key? do
      it "should return false when key does not exist" do
        subject.should_not have_key('abc')
      end
      it "should return true when key does exist" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set)
        subject.should have_key('x')
      end
    end

    describe :[] do
      it "should return nil for not existing value" do
        subject['x'].should be_nil
      end      
    end


    describe :current_transaction_sequence do
      it "should return 0 for no transactions" do
        subject.current_transaction_sequence.should == 0
      end      
      it "should return last transaction sequence for first update" do
        subject.update(1,{'abc' => "value"}, {'x' => 'abc'}, [].to_set)
        subject.current_transaction_sequence.should == 1
      end      
    end

  end
end