require 'dp_stm_map'

require 'thread'
require 'timeout'
require 'tmpdir'
require 'fileutils'
Dir["./spec/**/*_shared.rb"].sort.each {|f| require f}

module DpStmMap

  describe DistributedPersistentStmMap do

    before do
      @storage_dir=Dir.mktmpdir
      Dir.mkdir "%s/server" % @storage_dir
      Dir.mkdir "%s/local" % @storage_dir
      server.start
    end

    after do
      server.stop
      FileUtils.rm_rf(@storage_dir)
      subject.stop
    end

    let(:server) {Manager.new port, "%s/server" % @storage_dir}

    let(:port) {31337}

    subject { DistributedPersistentStmMap.new 'localhost', port, "%s/local" % @storage_dir }


    describe :map_behaviour do

      before do
        queue=Queue.new
        subject.on_connected do
          queue << "connected"
        end
        subject.start
        timeout(1) do
          queue.pop.should == "connected"
        end
      end


      it_behaves_like "StmMap"
    end    


    it "should reconnect to server that has rebooted" do
      queue=Queue.new
      subject.on_connected do
        queue << true
      end
      subject.start
    end

    describe :on_connected do
      it "should be executed when client did connect to the manager" do

        queue=Queue.new
        subject.on_disconnected do
          queue << "disconnected"
        end
        subject.on_connected do
          queue << "connected"
        end
        subject.start
        timeout(1) do
          queue.pop.should == "connected"
        end

        sleep 0.1
        server.stop        

        timeout(5) do
          queue.pop.should == "disconnected"
        end

        server.start

        timeout(5) do
          queue.pop.should == "connected"
        end

      end
    end

    describe :on_disconnected do
      it "should be executed when client did disconnect from the manager" do
        queue=Queue.new
        subject.on_disconnected do
          queue << "disconnected"
        end
        subject.on_connected do
          queue << "connected"
        end
        subject.start

        timeout(1) do
          queue.pop.should == "connected"
        end

        subject.stop

        timeout(1) do
          queue.pop.should == "disconnected"
        end
      end
    end

    describe :stop do
      it "should stop client" do
        queue=Queue.new
        subject.on_disconnected do
          queue << "disconnected"
        end
        subject.on_connected do
          queue << "connected"
        end
        subject.start
        timeout(1) do
          queue.pop.should == "connected"
        end
        subject.stop
        timeout(1) do
          queue.pop.should == "disconnected"
        end
      end
    end
  end
end
