require 'dp_stm_map'

module DpStmMap

  class SomeObject
    attr_accessor :value

    def == other
      case other
      when SomeObject
        other.value == self.value
      else
        false
      end
    end
  end

  describe ObjectStore do


    let(:some_object) {v=SomeObject.new; v.value='v1'; v}

    subject {ObjectStore.new InMemoryStmMap.new}


    describe :on_atomic do
      it "should be executed after atomic change" do
        changes=nil
        subject.on_atomic do |c|
          changes=c
        end
        subject.atomic do |tx|
          tx[:user,'abc']=some_object
        end
        changes.should == {[:user, 'abc'] => [nil, some_object]}
      end
    end



    describe :validate_atomic do
      it "should stop transaction when exception is raised" do

        changes=nil
        subject.validate_atomic do |c|
          changes=c
          raise "something went wrong"
        end

        expect do
          subject.atomic do |tx|
            tx[:user,'abc']=some_object
          end
        end.to raise_error "something went wrong"

        changes.should == {[:user, 'abc'] => [nil, some_object]}

        subject.atomic_read do |tx|
          tx[:user,'abc'].should == nil
        end

      end
    end

    describe :atomic do
      it "should store Ruby objects" do
        subject.atomic do |tx|
          tx[:user,'abc']=some_object
        end

        subject.atomic_read do |tx|
          tx.should have_key(:user, 'abc')
          tx[:user,'abc'].should == some_object
        end
      end
      it "should store nil values" do
        subject.atomic do |tx|
          tx[:user,'abc']=nil
        end

        subject.atomic_read do |tx|
          tx[:user,'abc'].should == nil
        end
      end
    end
  end
end
